searchData={"items":[{"type":"module","title":"LibRedis","doc":"# LibRedis\n\nLibRedis is an eloquently crafted Redis client implemented in Elixir. It serves as a remarkable encapsulation of the [Redix](https://github.com/whatyouhide/redix) project, while additionally incorporating connection pooling and Redis cluster capabilities. Within this client, two primary interfaces are exposed, namely `command/3` and `pipeline/3`.\n\nPlease refer to the detailed project documentation: [hexdocs](https://hexdocs.pm/lib_redis/0.1.1/LibRedis.html)","ref":"LibRedis.html"},{"type":"module","title":"WARNING: - LibRedis","doc":"**The deployment of the Cluster mode in a production environment necessitates meticulous deliberation and vigilance, as it is still undergoing comprehensive and stringent testing.**","ref":"LibRedis.html#module-warning"},{"type":"module","title":"Features - LibRedis","doc":"- [x] Connection pooling\n- [x] Standalone mode\n- [x] Cluster mode","ref":"LibRedis.html#module-features"},{"type":"module","title":"Installation - LibRedis","doc":"To use LibRedis in your Elixir project, simply add it as a dependency in your `mix.exs` file:\n\n```elixir\ndefp deps do\n  [\n    {:lib_redis, \"~> 0.1.3\"}\n  ]\nend\n```\n\nThen, run `mix deps.get` to install the dependency.","ref":"LibRedis.html#module-installation"},{"type":"module","title":"Usage - LibRedis","doc":"#","ref":"LibRedis.html#module-usage"},{"type":"module","title":"Standalone - LibRedis","doc":"To use LibRedis in singleton mode, you have to set your client as `:standalone` mode:\n\n```elixir\nstandalone_options = [\n  name: :redis,\n  mode: :standalone,\n  url: \"redis://:pwd@localhost:6379\",\n  pool_size: 5\n]\n\nstandalone = LibRedis.new(standalone_options)\n```\n\nThen, add the `LibRedis` application to your supervision tree:\n\n```elixir\nchildren = [\n  {LibRedis, redis: standalone}\n]\n```\n\nThe `options` parameter is a keyword list that contains the connection options for Redis. The `pool_size` option specifies the number of connections in the pool.\nThanks to powerful [nimble_pool](https://github.com/dashbitco/nimble_pool), we can easily use a nimble pool to manage redis connections.\n\nOnce you have a instance, you can use the `command/3` and `pipeline/3` APIs to execute Redis commands:\n\n```elixir\n{:ok, result} = LibRedis.command(standalone, [\"GET\", \"mykey\"])\n```\n\nThe `command/3` function takes a LibRedis object, a Redis command as a list of strings, and an optional timeout in milliseconds. It returns a tuple with the status of the command execution and the result.\n\n```elixir\n{:ok, result} = LibRedis.pipeline(standalone, [\n  [\"SET\", \"mykey\", \"value1\"],\n  [\"GET\", \"mykey\"]\n])\n```\n\nThe `pipeline/3` function takes a LibRedis object and a list of Redis commands, where each command is represented as a list of strings. It returns a tuple with the status of the pipeline execution and a list of results.\n\n#","ref":"LibRedis.html#module-standalone"},{"type":"module","title":"Redis Cluster - LibRedis","doc":"To use LibRedis with Redis cluster, first create a cluster using the `LibRedis.Cluster` module:\n\n```elixir\ncluster_options = [\n  name: :redis,\n  mode: :cluster,\n  url: \"redis://localhost:6381,redis://localhost:6382,redis://localhost:6383,redis://localhost:6384,redis://localhost:6385\",\n  password: \"123456\",\n  pool_size: 5\n]\n\ncluster = LibRedis.new(cluster_options)\n```\n\nThen, add the `LibRedis` application to your supervision tree:\n\n```elixir\nchildren = [\n  {LibRedis, redis: cluster}\n]\n```\n\nThe `cluster_options` parameter is a keyword list that contains the connection options for Redis cluster. \n\nOnce you have a cluster, you can use the `command/3` and `pipeline/3` APIs to execute Redis commands:\n\n```elixir\n{:ok, result} = LibRedis.command(cluster, [\"GET\", \"mykey\"])\n```\n\nThe `command/3` function takes a cluster object, a Redis command as a list of strings, and an optional timeout in milliseconds. It returns a tuple with the status of the command execution and the result.\n\n```elixir\n{:ok, result} = LibRedis.pipeline(cluster, [\n  [\"SET\", \"mykey\", \"value1\"],\n  [\"GET\", \"mykey\"]\n])\n```\n\nThe `pipeline/3` function takes a cluster object and a list of Redis commands, where each command is represented as a list of strings. It returns a tuple with the status of the pipeline execution and a list of results.","ref":"LibRedis.html#module-redis-cluster"},{"type":"module","title":"Design - LibRedis","doc":"#","ref":"LibRedis.html#module-design"},{"type":"module","title":"Connection Pooling - LibRedis","doc":"I leverage [nimble_pool](https://github.com/dashbitco/nimble_pool) to encapsulate Redix connections and administer them within a pool. Every time a command or pipeline operation is executed, a connection is acquired from the pool and subsequently returned upon completion of the operation.\n\n#","ref":"LibRedis.html#module-connection-pooling"},{"type":"module","title":"Standalone Mode - LibRedis","doc":"In standalone mode, the client maintains a solitary connection pool to a singular Redis instance. The client will seamlessly reestablish connection with the Redis instance in the event of any disruption.\n\n#","ref":"LibRedis.html#module-standalone-mode"},{"type":"module","title":"Cluster Mode - LibRedis","doc":"Cluster mode presents a significantly more intricate setup compared to standalone mode. Within this mode, the client executes the `CLUSTER SLOTS` command to obtain the cluster's topology. The client periodically retrieves this topology, typically every 10 seconds (though it can be adjusted), and stores it locally in the form of a cache (known as an Agent).\n\nThis cache facilitates the client's management of node addresses and their corresponding connections. If an address is not found within the cache, the client promptly establishes a new connection and stores it accordingly.\n\nThe client processes a single command by first identifying the node to which the command should be sent based on the command's key. It then locates the corresponding connection from the cache and executes the command on the connection.\n\nPipeline operation is far more complex than command operation. \n- first, client will group commands in pipelines by it's key;\n- then, client will iterate the grouped pipelines and execute them in parallel;\n- finnal, client will merge the results of the pipelines and return them to the caller.\n\nClient will automatically retry the command if the command fails due to a `Moved` error. If the command fails with a `Moved` error, means cluster topology has changed, the client will update the cluster topology and retry the command. If the command fails due to a Redis error, the client will return the error to the caller.","ref":"LibRedis.html#module-cluster-mode"},{"type":"module","title":"Test - LibRedis","doc":"To run unit tests, you can start a Redis service using Docker locally and then execute the `mix test` command.\n\nThe specific way to start the Redis service can be referred to in the run.md file under the docker-compose directory.\n\n```\nCompiling 8 files (.ex)\nGenerated lib_redis app\nCover compiling modules ...\nFinished in 3.3 seconds (0.00s async, 3.3s sync)\n32 tests, 0 failures\n\nRandomized with seed 498986\n\nGenerating cover results ...\n\n| Percentage  | Module                     |\n| ----------- | -------------------------- |\n| 0.00%       | LibRedis.Utils             |\n| 50.00%      | LibRedis.Error             |\n| 50.00%      | LibRedis.Standalone        |\n| 71.88%      | LibRedis.Pool              |\n| 76.47%      | LibRedis.SlotFinder        |\n| 77.91%      | LibRedis.Cluster           |\n| 93.75%      | LibRedis.ClientStore       |\n| 96.15%      | LibRedis                   |\n| 100.00%     | LibRedis.Client            |\n| 100.00%     | LibRedis.SlotStore         |\n| 100.00%     | LibRedis.Typespecs         |\n| ----------- | -------------------------- |\n| 80.71%      | Total                      |\n\nCoverage test failed, threshold not met:\n\n    Coverage:   80.71%\n    Threshold:  90.00%\n\nGenerated HTML coverage results in \"cover\" directory\n```","ref":"LibRedis.html#module-test"},{"type":"module","title":"License - LibRedis","doc":"LibRedis is released under the [MIT License](https://opensource.org/licenses/MIT).","ref":"LibRedis.html#module-license"},{"type":"function","title":"LibRedis.child_spec/1","doc":"","ref":"LibRedis.html#child_spec/1"},{"type":"function","title":"LibRedis.command/3","doc":"execute a redis command","ref":"LibRedis.html#command/3"},{"type":"function","title":"Examples - LibRedis.command/3","doc":"iex> LibRedis.command(redis, [\"SET\", \"key\", \"value\"])\n    {:ok, \"OK\"}","ref":"LibRedis.html#command/3-examples"},{"type":"function","title":"LibRedis.new/1","doc":"create a new redis client","ref":"LibRedis.html#new/1"},{"type":"function","title":"Options - LibRedis.new/1","doc":"* `:name` (`t:atom/0`) - The name of the redis client The default value is `:redis`.\n\n* `:mode` - The mode of the redis client, :standalone or :cluster The default value is `:standalone`.\n\n* `:url` (`t:String.t/0`) - Required. The url of the redis server, like redis://:123456@localhost:6379\n\n* `:password` (`t:String.t/0`) - The password of the redis server, this option is useful in cluster mode The default value is `\"\"`.\n\n* `:pool_size` (`t:non_neg_integer/0`) - The pool size of the redis client's pool The default value is `10`.","ref":"LibRedis.html#new/1-options"},{"type":"function","title":"Examples - LibRedis.new/1","doc":"iex> standalone_options = [\n      name: :redis,\n      mode: :standalone,\n      url: \"redis://:pwd@localhost:6379\",\n      pool_size: 5\n    ]\n    iex> LibRedis.new(standalone_options)","ref":"LibRedis.html#new/1-examples"},{"type":"function","title":"LibRedis.pipeline/3","doc":"execute a redis pipeline","ref":"LibRedis.html#pipeline/3"},{"type":"function","title":"Examples - LibRedis.pipeline/3","doc":"iex> LibRedis.pipeline(redis, [[\"SET\", \"key\", \"value\"], [\"GET\", \"key\"]])\n    {:ok, [\"OK\", \"value\"]}","ref":"LibRedis.html#pipeline/3-examples"},{"type":"function","title":"LibRedis.start_link/1","doc":"","ref":"LibRedis.html#start_link/1"},{"type":"type","title":"LibRedis.options/0","doc":"","ref":"LibRedis.html#t:options/0"},{"type":"type","title":"LibRedis.t/0","doc":"","ref":"LibRedis.html#t:t/0"},{"type":"behaviour","title":"LibRedis.Client","doc":"Redis client behaviour, there are 2 implementations of this behaviour:\n- LibRedis.Standalone\n- LibRedis.Cluster","ref":"LibRedis.Client.html"},{"type":"function","title":"LibRedis.Client.command/3","doc":"","ref":"LibRedis.Client.html#command/3"},{"type":"callback","title":"LibRedis.Client.command/3","doc":"","ref":"LibRedis.Client.html#c:command/3"},{"type":"callback","title":"LibRedis.Client.new/1","doc":"","ref":"LibRedis.Client.html#c:new/1"},{"type":"function","title":"LibRedis.Client.pipeline/3","doc":"","ref":"LibRedis.Client.html#pipeline/3"},{"type":"callback","title":"LibRedis.Client.pipeline/3","doc":"","ref":"LibRedis.Client.html#c:pipeline/3"},{"type":"callback","title":"LibRedis.Client.start_link/1","doc":"","ref":"LibRedis.Client.html#c:start_link/1"},{"type":"opaque","title":"LibRedis.Client.command_t/0","doc":"","ref":"LibRedis.Client.html#t:command_t/0"},{"type":"type","title":"LibRedis.Client.resp_t/0","doc":"","ref":"LibRedis.Client.html#t:resp_t/0"},{"type":"type","title":"LibRedis.Client.t/0","doc":"","ref":"LibRedis.Client.html#t:t/0"},{"type":"module","title":"LibRedis.ClientStore","doc":"redis client store, using registry to store {host, port} -> pid mapping","ref":"LibRedis.ClientStore.html"},{"type":"function","title":"LibRedis.ClientStore.child_spec/1","doc":"","ref":"LibRedis.ClientStore.html#child_spec/1"},{"type":"function","title":"LibRedis.ClientStore.get/2","doc":"","ref":"LibRedis.ClientStore.html#get/2"},{"type":"function","title":"LibRedis.ClientStore.new/1","doc":"","ref":"LibRedis.ClientStore.html#new/1"},{"type":"function","title":"LibRedis.ClientStore.random/1","doc":"","ref":"LibRedis.ClientStore.html#random/1"},{"type":"function","title":"LibRedis.ClientStore.start_link/1","doc":"","ref":"LibRedis.ClientStore.html#start_link/1"},{"type":"type","title":"LibRedis.ClientStore.k/0","doc":"","ref":"LibRedis.ClientStore.html#t:k/0"},{"type":"type","title":"LibRedis.ClientStore.opts/0","doc":"","ref":"LibRedis.ClientStore.html#t:opts/0"},{"type":"type","title":"LibRedis.ClientStore.t/0","doc":"","ref":"LibRedis.ClientStore.html#t:t/0"},{"type":"type","title":"LibRedis.ClientStore.v/0","doc":"","ref":"LibRedis.ClientStore.html#t:v/0"},{"type":"module","title":"LibRedis.Cluster","doc":"cluster redis client","ref":"LibRedis.Cluster.html"},{"type":"function","title":"LibRedis.Cluster.child_spec/1","doc":"","ref":"LibRedis.Cluster.html#child_spec/1"},{"type":"function","title":"LibRedis.Cluster.handle_call/3","doc":"","ref":"LibRedis.Cluster.html#handle_call/3"},{"type":"function","title":"LibRedis.Cluster.handle_info/2","doc":"","ref":"LibRedis.Cluster.html#handle_info/2"},{"type":"function","title":"LibRedis.Cluster.init/1","doc":"","ref":"LibRedis.Cluster.html#init/1"},{"type":"type","title":"LibRedis.Cluster.node_info/0","doc":"","ref":"LibRedis.Cluster.html#t:node_info/0"},{"type":"type","title":"LibRedis.Cluster.slot_info/0","doc":"","ref":"LibRedis.Cluster.html#t:slot_info/0"},{"type":"opaque","title":"LibRedis.Cluster.state/0","doc":"","ref":"LibRedis.Cluster.html#t:state/0"},{"type":"type","title":"LibRedis.Cluster.t/0","doc":"","ref":"LibRedis.Cluster.html#t:t/0"},{"type":"module","title":"LibRedis.Pool","doc":"","ref":"LibRedis.Pool.html"},{"type":"module","title":"LibRedis.Pool - LibRedis.Pool","doc":"Wrap [redix](https://github.com/whatyouhide/redix) with [nimble_pool](https://github.com/dashbitco/nimble_pool)\n\n#","ref":"LibRedis.Pool.html#module-libredis-pool"},{"type":"module","title":"Usage - LibRedis.Pool","doc":"```\niex> pool = LibRedis.Pool.new(name: :redis_pool, pool_size: 5, url: \"redis://:123456@localhost:6379\")\niex> {:ok, _} = LibRedis.Pool.start_link(pool: pool)\niex> LibRedis.Pool.command(pool, [\"SET\", \"foo\", \"bar\"])\n{:ok, \"OK\"}\niex> LibRedis.Pool.pipeline(pool, [[\"SET\", \"foo\", \"bar\"], [\"SET\", \"bar\", \"foo\"]])\n{:ok, [\"OK\", \"OK\"]]}\n```","ref":"LibRedis.Pool.html#module-usage"},{"type":"function","title":"LibRedis.Pool.child_spec/1","doc":"","ref":"LibRedis.Pool.html#child_spec/1"},{"type":"function","title":"LibRedis.Pool.command/3","doc":"delegate to Redix.command/2","ref":"LibRedis.Pool.html#command/3"},{"type":"function","title":"Examples - LibRedis.Pool.command/3","doc":"iex> LibRedis.Pool.command(pool, [\"SET\", \"foo\", \"bar\"])\n    {:ok, \"OK\"}","ref":"LibRedis.Pool.html#command/3-examples"},{"type":"function","title":"LibRedis.Pool.new/1","doc":"create new redis pool instance","ref":"LibRedis.Pool.html#new/1"},{"type":"function","title":"Options - LibRedis.Pool.new/1","doc":"* `:name` - The name of the pool The default value is `:redis_pool`.\n\n* `:pool_size` (`t:non_neg_integer/0`) - The size of the pool The default value is `10`.\n\n* `:url` (`t:String.t/0`) - Required. The url of the redis server, like redis://:123456@localhost:6379","ref":"LibRedis.Pool.html#new/1-options"},{"type":"function","title":"Examples - LibRedis.Pool.new/1","doc":"iex> LibRedis.Pool.new()","ref":"LibRedis.Pool.html#new/1-examples"},{"type":"function","title":"LibRedis.Pool.pipeline/3","doc":"delegate to Redix.pipeline/2","ref":"LibRedis.Pool.html#pipeline/3"},{"type":"function","title":"Examples - LibRedis.Pool.pipeline/3","doc":"iex> LibRedis.Pool.pipeline(pool, [[\"SET\", \"foo\", \"bar\"], [\"SET\", \"bar\", \"foo\"]])\n    {:ok, [\"OK\", \"OK\"]]}","ref":"LibRedis.Pool.html#pipeline/3-examples"},{"type":"function","title":"LibRedis.Pool.start_link/1","doc":"start redis with nimble pool","ref":"LibRedis.Pool.html#start_link/1"},{"type":"function","title":"Examples - LibRedis.Pool.start_link/1","doc":"iex> LibRedis.Pool.start_link(pool: LibRedis.Pool.new())","ref":"LibRedis.Pool.html#start_link/1-examples"},{"type":"type","title":"LibRedis.Pool.pool_opts_t/0","doc":"","ref":"LibRedis.Pool.html#t:pool_opts_t/0"},{"type":"type","title":"LibRedis.Pool.t/0","doc":"","ref":"LibRedis.Pool.html#t:t/0"},{"type":"module","title":"LibRedis.SlotFinder","doc":"This module is responsible for reloading the slot cache.","ref":"LibRedis.SlotFinder.html"},{"type":"function","title":"LibRedis.SlotFinder.hash_slot/1","doc":"","ref":"LibRedis.SlotFinder.html#hash_slot/1"},{"type":"module","title":"LibRedis.SlotStore","doc":"used to store slot topology","ref":"LibRedis.SlotStore.html"},{"type":"function","title":"LibRedis.SlotStore.child_spec/1","doc":"","ref":"LibRedis.SlotStore.html#child_spec/1"},{"type":"function","title":"LibRedis.SlotStore.get/1","doc":"get slots layer","ref":"LibRedis.SlotStore.html#get/1"},{"type":"function","title":"Examples - LibRedis.SlotStore.get/1","doc":"iex> s = LibRedis.SlotStore.Default.new()\n    iex> LibRedis.SlotStore.Default.get(s)","ref":"LibRedis.SlotStore.html#get/1-examples"},{"type":"function","title":"LibRedis.SlotStore.new/1","doc":"new storage\nthe only argument is `:name`, which is the name of the storage","ref":"LibRedis.SlotStore.html#new/1"},{"type":"function","title":"Examples - LibRedis.SlotStore.new/1","doc":"iex> LibRedis.SlotStore.Default.new(name: :slot_agent)","ref":"LibRedis.SlotStore.html#new/1-examples"},{"type":"function","title":"LibRedis.SlotStore.put/2","doc":"srore slots layer","ref":"LibRedis.SlotStore.html#put/2"},{"type":"function","title":"Examples - LibRedis.SlotStore.put/2","doc":"iex> s = LibRedis.SlotStore.Default.new()\n    iex> LibRedis.SlotStore.Default.put(s, [%{start_slot: 0, end_slot: 1}])\n    :ok","ref":"LibRedis.SlotStore.html#put/2-examples"},{"type":"function","title":"LibRedis.SlotStore.start_link/1","doc":"","ref":"LibRedis.SlotStore.html#start_link/1"},{"type":"opaque","title":"LibRedis.SlotStore.node_info/0","doc":"","ref":"LibRedis.SlotStore.html#t:node_info/0"},{"type":"opaque","title":"LibRedis.SlotStore.opts/0","doc":"","ref":"LibRedis.SlotStore.html#t:opts/0"},{"type":"opaque","title":"LibRedis.SlotStore.slot/0","doc":"","ref":"LibRedis.SlotStore.html#t:slot/0"},{"type":"type","title":"LibRedis.SlotStore.t/0","doc":"","ref":"LibRedis.SlotStore.html#t:t/0"},{"type":"module","title":"LibRedis.Standalone","doc":"Standalone redis client is just a delegate to LibRedis.Pool","ref":"LibRedis.Standalone.html"},{"type":"module","title":"LibRedis.Typespecs","doc":"Some typespecs for LibRedis","ref":"LibRedis.Typespecs.html"},{"type":"type","title":"LibRedis.Typespecs.command_t/0","doc":"","ref":"LibRedis.Typespecs.html#t:command_t/0"},{"type":"type","title":"LibRedis.Typespecs.name/0","doc":"","ref":"LibRedis.Typespecs.html#t:name/0"},{"type":"type","title":"LibRedis.Typespecs.node_info/0","doc":"","ref":"LibRedis.Typespecs.html#t:node_info/0"},{"type":"type","title":"LibRedis.Typespecs.on_start/0","doc":"","ref":"LibRedis.Typespecs.html#t:on_start/0"},{"type":"type","title":"LibRedis.Typespecs.password/0","doc":"","ref":"LibRedis.Typespecs.html#t:password/0"},{"type":"type","title":"LibRedis.Typespecs.slot/0","doc":"","ref":"LibRedis.Typespecs.html#t:slot/0"},{"type":"type","title":"LibRedis.Typespecs.url/0","doc":"","ref":"LibRedis.Typespecs.html#t:url/0"},{"type":"module","title":"LibRedis.Utils","doc":"some tools","ref":"LibRedis.Utils.html"},{"type":"function","title":"LibRedis.Utils.debug/1","doc":"","ref":"LibRedis.Utils.html#debug/1"},{"type":"exception","title":"LibRedis.Error","doc":"redis error","ref":"LibRedis.Error.html"},{"type":"function","title":"LibRedis.Error.new/1","doc":"","ref":"LibRedis.Error.html#new/1"},{"type":"type","title":"LibRedis.Error.t/0","doc":"","ref":"LibRedis.Error.html#t:t/0"}],"content_type":"text/markdown"}